/**
* Filename: Event.java (in org.openbandy.event)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/


package org.openbandy.event;


/**
 * This is the representation of an Event. Note that every Event
 * has a <b>validity </b>. Events in storage which are older than
 * (creationTime + validity) will be removed from storage. Events where
 * validity = INFINIT_VALIDITY will remain.
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 * 
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.8
 * 
 */
public class Event {
    
    /* Constants */
    protected static final int INFINIT_VALIDITY = 0;
    public static final int DEFAULT_VALIDITY = INFINIT_VALIDITY;
	
	/* timestamp indicating when the event has been created */
	protected long creationTime = 0;
	
	/* The events validity in minutes */
	protected int validity = DEFAULT_VALIDITY;
	
	public Event(){
	    creationTime = System.currentTimeMillis();
	}
	
	/**
	 * @return Returns a timestamp when the event has been created.
	 */
	public long getCreationTime() {
		return creationTime;
	}
	/**
	 * @return Returns an int value indicating for how many minutes
	 * 		   the event will be valid.
	 */
	public int getValidity() {
		return validity;
	}

    /**
     * TODO comment!
     *
     * @param validity
     */
    public void setValidity(int validity) {
        this.validity = validity;
    }

}
