/**
* Filename: EventSource.java (in org.openbandy.event)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.event;

import java.util.Vector;

import org.openbandy.service.LogService;

/**
 * TODO describe purpose and usage
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 * 
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public abstract class EventSource extends Thread {
    
    /**
     * State of the current thread.
     */
    protected volatile boolean running = false;
    
    /**
     *  Class holding the reference to this EventSource.
     */
    protected EventSourceOwner owner;
    
    /**
     *  A list of all types of Events that are currently
     *  consumed by at least one EventSink.
     */
    protected Vector consumedEvents;
    
    /**
     *  Constructor
     */
    public EventSource(String name, EventSourceOwner owner){
        super(name);
        
        if(owner != null){
        	this.owner = owner;
        }
        else {
        	LogService.warn(this, "Owner must not be null!");
        }
        
        this.consumedEvents = new Vector();
    }
    
    
    /* *************** Getter and Setter ***************** */
    
    /**
     * @return True if this Source is running.
     */
    public boolean isRunning(){
        return running;
    }
    
    /**
     *  The class owning this source, i.e., having a reference
     *  to it, must update this reference as the source was
     *  newly instantiated.
     */
    protected void updateReferenceAtOwner(){
        owner.setNewInstance(this);
    }
    
    
    /* *************** EventSource Methods ***************** */
    
    public void startEventSource(String eventShortClassName){
        if(getProvidedEventTypes().contains(eventShortClassName)){
            consumedEvents.addElement(eventShortClassName);
            
            if(! this.running){
                this.start();
                LogService.debug(this, "Started EventSource : "+getName());
            }
        }
    }
    
    /**
     * Starts the EventSource.
     */
    public abstract void run();
    
    /**
     * While registering at the EventRouter, the latter will request
     * the EventSource to state the types of Events it will delegate.
     * 
     * @return A Vector of Strings containing the short class names of the Events.
     */
    public abstract Vector getProvidedEventTypes();
    
    /**
     * As the EventRouter must eventually create new instances of
     * an EventSource in order to restart it, an EventSource must
     * provide this method which must create (and return) a new
     * instance and subsequently reset its reference in the
     * class owning it.
     */
    public abstract EventSource newlyInstantiate();
    
    /**
     * Advises the EventSource, that events of type
     * <code>eventShortClassName</code> will no longer
     * be conumsed.
     * If non of the provided event types is consumed,
     * stop the event source thread.
     * 
     * @param eventShortClassName
     */
    public abstract void stopEventSource(String eventShortClassName);
    
    /**
     * Stops the source.
     */
    public abstract void stopEventSource();
}
