/**
* Filename: EventSourceController.java (in org.openbandy.event)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.event;

import java.util.Enumeration;
import java.util.Hashtable;


/**
 * TODO describe purpose and usage
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 * 
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public class EventSourceController {
    
    /* the event source broker */
    private static EventSourceBroker eventSourceBroker;
    
    /* list of registered sources (only index number in EventSourceBroker */
    private Hashtable registeredEventSources;
    
    /* short class name of event type controlled */
    private String eventShortClassName;
    
    /* Constructor */
    public EventSourceController(String eventShortClassName, EventSourceBroker eventSourceBroker){
        this.eventShortClassName = eventShortClassName;
        EventSourceController.eventSourceBroker = eventSourceBroker;
        this.registeredEventSources = new Hashtable();        
    }
    
    //TODO remove dev code
    public Hashtable getRegisteredSources(){
        return registeredEventSources;
    }
    
    /* *************** Getter and Setter ***************** */
    
    public String getEventShortClassName(){
        return eventShortClassName;
    }
    
    public void addSource(Integer eventSourceIndex){
        /* avoid duplicate entries */
        registeredEventSources.put(eventSourceIndex, eventSourceIndex);
    }
    
    public boolean removeSource(Integer eventSourceIndex){
        return (registeredEventSources.remove(eventSourceIndex) != null);
    }
    
    public boolean isEmpty(){
        return (registeredEventSources.size() == 0);
    }
    
    /* *************** Worker Methods ***************** */
    
    /**
     * Checks if every controlled event source is running.
     * If a source is not running yet, start it.
     */
    public synchronized void ensureRunning(){  
        Enumeration elements = registeredEventSources.elements();
        while(elements.hasMoreElements()){
            Integer eventSourceIndex = (Integer)elements.nextElement();
            
            /* get the corresponding event source */
            EventSource eventSource = eventSourceBroker.getEventSource(eventSourceIndex);
            
            /* start if necessary */
            if(eventSource != null){
                if(! eventSource.isRunning()){
                    try{
                        eventSource.startEventSource(eventShortClassName);
                    }catch(IllegalThreadStateException itse){
                        
                        /* start new instance of event source */
                        EventSource newEventSource = eventSourceBroker.getNewEventSourceInstance(eventSourceIndex);
                        newEventSource.startEventSource(eventShortClassName);
                    }
                }
            }
        }
    }
    
    public synchronized void stopSources(){
        Enumeration elements = registeredEventSources.elements();
        while(elements.hasMoreElements()){
            Integer eventSourceIndex = (Integer)elements.nextElement();
            
            /* get the corresponding event source */
            EventSource eventSource = eventSourceBroker.getEventSource(eventSourceIndex);
            
            /* stop it */
            if(eventSource.isRunning()){
                eventSource.stopEventSource(eventShortClassName);
            }
        }
    }
}
