/**
* Filename: TestData.java (in org.openbandy.example.test)
* This file is part of the OpenBandy project.
*
* www.openbandy.org
*/

package org.openbandy.example.test;

import org.openbandy.example.Car;
import org.openbandy.example.Crankshaft;
import org.openbandy.example.Engine;
import org.openbandy.example.EuropeanSportscar;
import org.openbandy.example.Sportscar;
import org.openbandy.example.Tire;

/**
 * Provides static method(s) that generate serializable objects
 * used for testing purposes.
 *
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 * 
 */
public final class TestDataGenerator {
	
	public static void fillCarObjectWithDummyValues(Car car){
		
		/* first we create a new instance of 'Crankshaft.java' */
		Crankshaft crankshaft = new Crankshaft();
		crankshaft.setMaterial("alloy steel");
		crankshaft.setWeight(2600);
		
		/* create an instance of 'Engine.java' */
		Engine engine = new Engine();
		engine.setLayout("Boxer");
		engine.setCubicCapacity(3185);
		engine.setCylinders(6);

		/* add the crankshaft to the engine */
		engine.setCrankshaft(crankshaft);
		
		/* then we create four instances of 'Tire.java' */
		Tire frontLeftTire = new Tire();
		frontLeftTire.setDimension("245");
		frontLeftTire.setMountedAt("front left");
		
		Tire frontRightTire = new Tire();
		frontRightTire.setDimension("245");
		frontRightTire.setMountedAt("front right");
		
		Tire rearLeftTire = new Tire();
		rearLeftTire.setDimension("265");
		rearLeftTire.setMountedAt("rear left");
		
		Tire rearRightTire = new Tire();
		rearRightTire.setDimension("265");
		rearRightTire.setMountedAt("rear right");

		
		/* now we set the basic attribute values of a car object */
		car.setBrand("Porsche");
		car.setLength(425);
		
		/* add the engine to the car */
		car.setEngine(engine);
		
		/*  finally we add the tires to the car */
		car.getTires().add(frontLeftTire);
		car.getTires().add(frontRightTire);
		car.getTires().add(rearLeftTire);
		car.getTires().add(rearRightTire);
	}
	
	/**
	 * TODO comment!
	 *
	 * @return
	 */
	public static Car createDummyCar(){
		Car car = new Car();
		
		fillCarObjectWithDummyValues(car);
		
		return car;
	}
	
	/**
	 * TODO comment!
	 *
	 * @return
	 */
	public static Sportscar createDummySportscar(){
		Sportscar sportscar = new Sportscar();
		
		fillCarObjectWithDummyValues(sportscar);
		
		/* set sportscar attributes */
		sportscar.setBhp(320);
		sportscar.setSprint(5.1);
		
		/* alter some car attributes */
		sportscar.setLength(398);
		
		return sportscar;
		
	}
	
	public static EuropeanSportscar createDummyEuropeanSportscar(){
		EuropeanSportscar europeanSportscar = new EuropeanSportscar();
		
		fillCarObjectWithDummyValues(europeanSportscar);
		
		/* set european sports car attributes */
		europeanSportscar.setNcap(4);
		
		/* alter the sports car attributes */
		europeanSportscar.setBhp(350);
		europeanSportscar.setSprint(4.9);
		
		/* alter some car attributes */
		europeanSportscar.setLength(402);
		
		return europeanSportscar;
	}

}
