/**
 * Filename: SerializableFactory.java (in org.openbandy.io)
 * This is part of the OpenBandy project.

 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.5
 * 
 * (c) Copyright 2007, ALL RIGHTS RESERVED.
 *
 * This factory provides statics methods to instantiate serializable objects.
 */

package org.openbandy.io;

import org.openbandy.service.LogService;

public class SerializableFactory {

	/**
	 * Creates a new instance of the class with the given name
	 * and returns it as <code>Serializable</code>
	 * 
	 * @param className Fully qualified class name
	 * @return New instance of this type
	 */
	public static Serializable getNewSerializableObject(String className){
		try{

			Serializable serializable = (Serializable)getNewObject(className);
			return serializable;

		}catch(ClassCastException cce){
			LogService.error(SerializableFactory.class, "Object is not a  Serializable: ", cce);
		}

		return null;
	}

	/**
	 * Creates a new instance of the class with the given name. 
	 * 
	 * @param className Fully qualified class name
	 * @return New instance of this type
	 */
	public static Object getNewObject(String className){
		if( (className != null) && (className != "") ) {
			try{

				Class persClass = Class.forName(className);
				return persClass.newInstance();

			}catch(ClassNotFoundException cnfe){
				LogService.error(SerializableFactory.class, "Class to create not found : ", cnfe);
			}catch(IllegalAccessException iae){
				LogService.error(SerializableFactory.class, "Could not create new object : ", iae);
			}catch(InstantiationException ie){
				LogService.error(SerializableFactory.class, "Could not instantiate new object. Make sure the serializable object features an argumentless constructor. ", ie);
			}
		}
		else {
			LogService.error("SerializableFactory", "No class name provided", null);
		}
		return null;
	}

}
