/**
 * Filename: RmsStorage.java (in org.openbandy.io.rms)
 * This is part of the OpenBandy project.
 * 
 * www.openbandy.org
 */

package org.openbandy.io.rms;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import javax.microedition.rms.RecordStore;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializationIdentifiers;
import org.openbandy.io.storage.Storage;
import org.openbandy.service.LogService;
import org.openbandy.util.StringUtil;


/**
 * Abtstract and encapsule rms knowlegede (id generation etc.)
 * 
 * TODO berprfe alle RmsConnections, wo kann ich sparen ??!!!!!
 * 
 * TODO NAMEN vereinfachen und konsequenter verwenden !!
 *
 * <br><br>
 * (c) Copyright P. Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 1.0
 */
public class RmsStorage extends Storage{

	/* Constants */    
	public static final String TYPE_POSTFIX = "s";
	protected static final String TYPES_RECORDSTORE_NAME = "TypesRecordStore";

	private static RmsStorage instance;

	/* Hasthable of rms ids, i.e. ids in the record store, recordStoreName -> id */
	private static Hashtable rmsIds;

	private static RmsConnection typesRecordStoreConnection;

	private RmsStorage(){
		typesRecordStoreConnection = getRmsConnection(TYPES_RECORDSTORE_NAME);
		if(typesRecordStoreConnection != null){
			typesRecordStoreConnection.connectAndCreateIfNecessary();
			typesRecordStoreConnection.setupAsRecordStoreWithHeader();
		}else{
			LogService.warn(this, "No connection for "+TYPES_RECORDSTORE_NAME);
		}

		rmsIds = new Hashtable();

		/* rebuild the hashtable from RMS */
		rebuildObjectsTable();
	}

	/**
	 * TODO comment!
	 *
	 * @return
	 */
	public static synchronized RmsStorage getInstance(){
		if (instance==null) {
			instance = new RmsStorage();
		}
		return instance;
	}

	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.Storage#existsInStorage(org.openbandy.io.Serializable)
	 */
	public int existsInStorage(Serializable serializable) {
		if(! SerializationIdentifiers.isValidId(serializable.getIdValue())){
			return 0;
		}
		String recordStoreName = getRecordStoreName(serializable);
		if( connections.containsKey(recordStoreName) ){
			return serializable.getIdValue();
		}
		return 0;
	}

	/**
	 * This method will create a new entry in the RMS for the
	 * serializable object. If the object already exists, the
	 * connection from the cache will be returned.
	 * If a new entry is created, the object will be assigned
	 * the corresponding id value.
	 * 
	 * @param serializable
	 * @return
	 */
	public synchronized RmsConnection addObject(Serializable serializable) {
		String objectRecordStoreName = getRecordStoreName(serializable);

		/* if the object already exists in storage, return existing connection */
		if(connections.containsKey(objectRecordStoreName)){
			RmsConnection conn = (RmsConnection)connections.get(objectRecordStoreName);
			return conn;
		}

		/* object has no id, create and assign a new one */
		String typeName = getTypeRecordStoreName(serializable);

		/* if it is the first object of this type, create typeRecordStore */
		RmsConnection typeConn = null;
		RmsTypeRecordStoreHeader typeRecordStoreHeader;
		if(!connections.containsKey(typeName)){
			typeConn = createTypeRecordStore(typeName);
			typeRecordStoreHeader = new RmsTypeRecordStoreHeader();
		}else{
			typeConn = (RmsConnection)connections.get(typeName);
			typeRecordStoreHeader = typeConn.getHeader();
		}

		//TODO das liesse sich mit dem 'types record store' part in createTypeRecordStore vereinen...    
		int objectIdInTypeRecordStore = typeConn.getNewIdFromRecordStore();
		typeRecordStoreHeader.addId(objectIdInTypeRecordStore);

		/* if the object has no valid id, not even a db id, assign a new rms id */
		if(! SerializationIdentifiers.isValidId(serializable.getIdValue())){

			int newObjectId = SerializationIdentifiers.RMS_ID_MIN+objectIdInTypeRecordStore;

			/* update objects id */
			serializable.setIdValue(newObjectId);
			LogService.debug(this, "Generated new rms id : "+serializable.getIdName()+"="+serializable.getIdValue());
		}

		objectRecordStoreName = getRecordStoreName(serializable);

		/* add this new entry in the corresponding typeRecordStore and update the header field */
		typeConn.updateStringAt(objectIdInTypeRecordStore, objectRecordStoreName);
		typeConn.updateStringAt(1, typeRecordStoreHeader.getHeader());

		/* create a new connection */
		RmsConnection objectConn = getRmsConnection(objectRecordStoreName);

		Vector objectAttributes = getObjectAttributes(serializable);
		objectConn.createEmptyRecordStore(objectRecordStoreName, objectAttributes.size());

		if(objectConn != null){
			/* add to cache */
			rmsIds.put(objectRecordStoreName, new Integer(objectIdInTypeRecordStore));
		}else{
			LogService.warn(this, "No connection for object "+serializable.getLegibleString());
		}

		return objectConn;
	}

	/**
	 * TODO comment!
	 *
	 * @param serializable
	 * @return
	 */
	public static RmsConnection getConnectionForObject(Serializable serializable){
		String objectRecordStoreName = getRecordStoreName(serializable);
		return getConnection(objectRecordStoreName);
	}

	/**
	 * TODO comment!
	 *
	 * @param objectRecordStoreName
	 * @return
	 */
	public static RmsConnection getConnection(String objectRecordStoreName){
		if(connections.containsKey(objectRecordStoreName)){
			return (RmsConnection)connections.get(objectRecordStoreName);
		}
		return null;
	}

	/**
	 * TODO comment!
	 *
	 * @param serializable
	 * @return
	 */
	public static String getRecordStoreName(Serializable serializable){
		return StringUtil.getShortClassName(serializable)+serializable.getIdValue();
	}

	/**
	 * TODO comment!
	 *
	 * @param serializable
	 * @return
	 */
	public static String getTypeRecordStoreName(Serializable serializable){
		return StringUtil.getShortClassName(serializable)+TYPE_POSTFIX;
	}


	/* ***************** Helper Methods ***************** */

	/**
	 * TODO comment!
	 *
	 * @param serializable
	 * @return
	 */
	public static Vector getObjectAttributes(Serializable serializable){
		Vector objectAttributes = new Vector();
		objectAttributes.removeAllElements();
		serializable.getAttributeNames(objectAttributes);
		serializable.getReferencedIdNames(objectAttributes);

		return objectAttributes;
	}

	private RmsConnection getRmsConnection(String name){
		if(connections.containsKey(name)){
			return (RmsConnection)connections.get(name);
		}else{
			RmsConnection conn = new RmsConnection(name);
			if(addConnection(conn)){
				return conn;
			}
		}
		return null;
	}

	private RmsConnection createTypeRecordStore(String typeRecordStoreName){
		RmsConnection typeConn = getRmsConnection(typeRecordStoreName);
		if(typeConn != null){

			typeConn.createEmptyRecordStore(typeRecordStoreName, 0);
			typeConn.setupAsRecordStoreWithHeader();

			/* add to types record store and update header */

			/* get new id */
			int typesId = typesRecordStoreConnection.getNewIdFromRecordStore();

			/* add to header and to cache */
			RmsTypeRecordStoreHeader trsh = typesRecordStoreConnection.getHeader();
			trsh.addId(typesId);
			rmsIds.put(typeRecordStoreName, new Integer(typesId));

			/* save value at new id an new header */
			typesRecordStoreConnection.updateStringAt(typesId, typeRecordStoreName);
			typesRecordStoreConnection.updateStringAt(1, trsh.getHeader());

			return typeConn;
		}else{
			LogService.warn(this, "No connection for "+typeRecordStoreName);
			return null;
		}
	}


	/**
	 * Rebuild the Bandy RMS Storage structure including
	 * the type recordstore.
	 */
	private void rebuildObjectsTable(){
		LogService.debug(this, "Rebuilding Caches");

		/* access types record store */
		RmsRecord types = new RmsRecord(typesRecordStoreConnection.getRecordStore());

		/* read header of types record store */
		types.moveTo(1);
		String headerString = types.readUTF(); 
		if(RmsTypeRecordStoreHeader.isValidHeaderString(headerString)){
			RmsTypeRecordStoreHeader typesStoreHeader = new RmsTypeRecordStoreHeader();
			typesStoreHeader.setHeader(headerString);

			Enumeration elements = typesStoreHeader.getTypeRecordStoreIds();
			while(elements.hasMoreElements()){
				String typeIdString = (String)elements.nextElement();
				int typeId = Integer.parseInt(typeIdString);
				types.moveTo(typeId);

				/* read name */
				String typeRecordStoreName = types.readUTF();

				/* create and open the corresponding connection */
				RmsConnection typeConnection = getRmsConnection(typeRecordStoreName);
				typeConnection.setHasHeader();

				if(typeConnection != null){
					typeConnection.connect();
					rmsIds.put(typeRecordStoreName, new Integer(typeId));

					LogService.debug(this, typeRecordStoreName+" already in store:");

					RmsRecord objects = new RmsRecord(typeConnection.getRecordStore());

					/* read the first Record, i.e. the header of the type record store */
					objects.moveTo(1);
					String typeHeaderString = objects.readUTF();

					/* handle type record stores */
					if(RmsTypeRecordStoreHeader.isValidHeaderString(typeHeaderString)){

						RmsTypeRecordStoreHeader trsh = new RmsTypeRecordStoreHeader();
						trsh.setHeader(typeHeaderString);

						Enumeration typeRecordStoreIds = trsh.getTypeRecordStoreIds();
						while(typeRecordStoreIds.hasMoreElements()){
							String objectIdString = (String)typeRecordStoreIds.nextElement();
							int id = Integer.parseInt(objectIdString);

							objects.moveTo(id);

							/* read entry name */
							String objectEntryName = objects.readUTF();

							/* create and open the corresponding connection */
							RmsConnection objectConnection = getRmsConnection(objectEntryName);
							objectConnection.connect();
							rmsIds.put(objectEntryName, new Integer(id));

							LogService.debug(this, "- "+objectEntryName);
						}
					}else{
						LogService.warn(this, "Invalid type record store: header missing");
					}
				}else{
					LogService.warn(this, "No connection for "+typeRecordStoreName);
				}	
			}
		}else{
			LogService.warn(this, "Did not rebuild objects table: no valid header");
		}
	}

	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.Storage#reset()
	 */
	public void reset(){
		/* close all RecordStores */
		closeAllConnections();

		/* delete all RecordStores */
		deleteAllRecordStores();

		instance = null;
	}

	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.Storage#isEmpty()
	 */
	public boolean isEmpty(){
//		TODO implement the real meaning of this method..
		/* actually quite simple solution, preferences and settings use 15 ids,
		 * so if not more than that is used, nothing else is stored */
		return (rmsIds.size() <= 15);
	}

	/**
	 * TODO comment!
	 *
	 * @return
	 */
	public static int numberOfUsedRecordStores(){
		String[] stores = RecordStore.listRecordStores();
		return stores.length;
	}

	private void deleteAllRecordStores(){
		String[] stores = RecordStore.listRecordStores();
		if(stores != null){
			for(int i=0; i<stores.length; i++){
				RmsConnection.deleteRecordStore(stores[i]);
			}
		}
		rmsIds.clear();
		instance = new RmsStorage();
		LogService.info(this, "Deleted ALL RecordStores");
	}

	/* (non-Javadoc)
	 * @see org.openbandy.io.storage.Storage#deleteShallow(org.openbandy.io.Serializable)
	 */
	public boolean deleteShallow(Serializable serializable) {

		/* delete corresponding objectRecordStore */
		String objectRecordStoreName = RmsStorage.getRecordStoreName(serializable);
		RmsConnection objectConn = (RmsConnection)connections.get(objectRecordStoreName);
		if(objectConn != null){

			/* delete object and remove connections */
			if(objectConn.executeDELETE(serializable)){
				/* remove connection from storage */
				removeConnection(objectConn);

				/* delete entry in type record store */
				deleteEntryInTypeRecordStore(serializable, objectRecordStoreName);

				/* remove entry from id cache */
				rmsIds.remove(objectConn.getRecordStoreName());

				return true;
			}
		}
		LogService.warn(this, "Object deletion failed: "+serializable.getLegibleString());
		return false;
	}

	private void deleteEntryInTypeRecordStore(Serializable serializable, String objectRecordStoreName){

		/* get connection for correspoding type record store */
		String typeRecordStoreName = getTypeRecordStoreName(serializable);
		RmsConnection typeConn =(RmsConnection)connections.get(typeRecordStoreName);

		deleteEntryFromRecordStoreWithHeader(typeConn, objectRecordStoreName);

		/* if no objects (except the header entry) of this type are in rms anymore, delete typeRecordStore */
		if(typeConn.getNumberOfRecords() == 1){

			/* delete types record store and remove connections */
			if(typeConn.isConnected()){
				typeConn.close();
			}
			RmsConnection.deleteRecordStore(typeRecordStoreName);
			removeConnection(typeConn);

			/* delete entry from typesRecordStore */
			deleteEntryFromRecordStoreWithHeader(typesRecordStoreConnection, typeRecordStoreName);

			/* remove entry from id cache */
			rmsIds.remove(typeRecordStoreName);
		}
	}

	private void deleteEntryFromRecordStoreWithHeader(RmsConnection rmsConnection, String recordStoreName){

		/* read header field */
		RmsRecord typeRecord = new RmsRecord(rmsConnection.getRecordStore());
		typeRecord.moveTo(1);
		String headerString = typeRecord.readUTF();

		RmsTypeRecordStoreHeader trsh = new RmsTypeRecordStoreHeader();
		trsh.setHeader(headerString);

		/* adjust header field */
		int idInTypeRecordStore = ((Integer)rmsIds.get(recordStoreName)).intValue();
		trsh.removeId(idInTypeRecordStore);

		/* write header field */
		rmsConnection.updateStringAt(1, trsh.getHeader());

		/* delete the actual entry */
		rmsConnection.deleteRecord(idInTypeRecordStore);
	}

	/* ************ Method for Developing Purposes only ************* */
	//TODO bessere comments, so dass sofort klar was hier passiert bzw. dargestellt wird
	/**
	 * TODO comment!
	 *
	 */
	public void printAllRecordStores(){
		//TODO muss umgebaut werden, da Preferences und LogStorage das RMS nicht so bentzen, wie hier vorgesehen..
		System.out.println("+++++++++++++++ RMS +++++++++++++++");

		System.out.println("--------------- ID Cache (size:"+rmsIds.size()+")---------------");
		for(Enumeration e = rmsIds.keys(); e.hasMoreElements();){
			String key = (String)e.nextElement();
			System.out.println(key+" key:"+rmsIds.get(key));
		}

		System.out.println("");
		System.out.println("");
		System.out.println("--------------- STORES ---------------");
		System.out.println("");

		String[] stores = RecordStore.listRecordStores();
		if(stores != null){
			/* only print, if more than just the TYPES_RECODSTORE record is in RMS */
			if(stores.length > 1){
				for(int i=0; i<stores.length; i++){
					System.out.print("RecordStore "+i+": "+stores[i]);
					RmsConnection co = getRmsConnection(stores[i]);
					if(co != null){
						co.connect();
						co.printRecordStore();
					}
					System.out.println("");
				}
			}
		}
	}
}
