/**
* Filename: StorageSerializationBuffer.java (in org.openbandy.io.storage)
* This is part of the OpenBandy project.

* @author Philipp Bolliger (philipp@bolliger.name)
* @version 1.0
* 
* (c) Copyright 2007, ALL RIGHTS RESERVED.
*
* Storage specific extenstion to the SerializationBuffer. In addition
* to the SerializationBuffer, this class has a hashtable of every
* subType as this is neede by the Database- and the Rms-Serializer.
*/

package org.openbandy.io.storage;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.openbandy.io.Serializable;
import org.openbandy.io.SerializationBuffer;
import org.openbandy.service.LogService;
import org.openbandy.util.CollectionsUtil;

public class StorageSerializationBuffer extends SerializationBuffer {

	/* Hashtable superTypeCopy->subType to update id values accordingly */
	protected Hashtable subTypes;

	/* Constructor */
	public StorageSerializationBuffer(){
		super();
		subTypes = new Hashtable();
	}


	/* *********** Getter and Setter ************ */

    /**
     * TODO comment!
     *
     * @return
     */
    public Vector getAllObjectEntriesInReverseOrder(){
    	
    	/* create a vector containing every objectEntry
    	 * within the known containers */
    	Vector objectEntries = new Vector();
    	Enumeration elements = valueContainers.elements();
    	while(elements.hasMoreElements()){
    		try {
				StorageValueContainer container = (StorageValueContainer)elements.nextElement();
				objectEntries.addElement(container.objectEntry);
			} catch (ClassCastException cce) {
				LogService.error(this, "ValueContainer is not a StorageValueContainer", cce);
			}
    	}
    	/* reverse Order */
        objectEntries = CollectionsUtil.reverseOrder(objectEntries);
    	return objectEntries;
    }
	
	/**
	 * TODO comment!
	 *
	 * @param superType
	 * @param subType
	 */
	public void addSubTypeRelation(Serializable superType, Serializable subType){
		subTypes.put(superType, subType);
	}

	/**
	 * TODO comment!
	 *
	 * @param superType
	 * @return
	 */
	public Serializable getSubType(Serializable superType){
		return (Serializable)subTypes.get(superType);
	}

	/**
	 * TODO comment!
	 *
	 * @param serializable
	 * @return
	 */
	public boolean containsSubType(Serializable serializable){
		return subTypes.contains(serializable);
	}

	/**
	 * TODO comment!
	 *
	 * @param serializable
	 * @return
	 */
	public boolean isSuperCopy(Serializable serializable){
		return subTypes.containsKey(serializable);
	}
}
