/**
* Filename: SmsScanner.java (in org.openbandy.net.client)
* This file is part of the BandyBasics project.
*
* www.openbandy.org
*/

package org.openbandy.net.client;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.util.Vector;

import javax.microedition.io.Connector;
import javax.wireless.messaging.BinaryMessage;
import javax.wireless.messaging.MessageConnection;

import org.openbandy.event.EventSource;
import org.openbandy.event.EventSourceOwner;
import org.openbandy.io.Message;
import org.openbandy.io.stream.StreamSerializer;
import org.openbandy.io.stream.StreamSerializerImpl;
import org.openbandy.net.MessageBroker;
import org.openbandy.service.LogService;

/**
 * TODO describe purpose and usage
 * 
 * <br><br>
 * (c) Copyright Philipp Bolliger 2007, ALL RIGHTS RESERVED.
 *
 * @author Philipp Bolliger (philipp@bolliger.name)
 * @version 0.1
 * 
 */
public class SmsScanner extends EventSource {
    
    /* stream serializer */
    private StreamSerializer serializer = null;
    
    /* client connection */
    private MessageConnection clientConnection = null;
    
    /* constructor */
    public SmsScanner(EventSourceOwner owner){
        super("ClientSmsScanner", owner);
        serializer = new StreamSerializerImpl();
    }
    
    
    /* ********** Methods for EventSource ************ */
    
    public Vector getProvidedEventTypes() {
        Vector providedEventTypes = new Vector();
        providedEventTypes.addElement("IncomingMessageEvent");
        return providedEventTypes;
    }
    
    public EventSource newlyInstantiate() {
        SmsScanner newInstance = new SmsScanner(this.owner);
        owner.setNewInstance(newInstance);
        return newInstance;
    }

    public void run() {
        
        /* variables */
        BinaryMessage binaryMessage = null;
        byte[] buffer = null;
        String address = "sms://:"+MessageBroker.SMS_PORT;
        String sender = "";
        
        /* start */
        running = true;

        while(running){
            try{
                /* open BLOCKING connection */
                clientConnection = (MessageConnection)Connector.open(address);
                LogService.debug(this, "Opened Connection "+address);
                
                /* wait for incoming binary sms messages */
                binaryMessage = (BinaryMessage)clientConnection.receive();
                sender = binaryMessage.getAddress();
                
                /* read binary sms message payload */
                buffer = binaryMessage.getPayloadData();
                
                /* deserialize message */
                ByteArrayInputStream bais = new ByteArrayInputStream(buffer);
                Message newMessage = (Message) serializer.getObjectFromXmlStream(bais);
                
                /* new message from server, put in queue and fire event */
                LogService.info(this, "Message from "+sender+": "+newMessage.getLegibleString());
                MessageScannerHelper.processNewMessage(newMessage);
                
                /* close connection */
                clientConnection.close();
                
            }catch(ClassCastException cce){
            	LogService.error(this, "Incoming Message is not BinaryMessage", cce);
            }catch(SecurityException se){
                /* the user did not allow to open port */
            	LogService.warn(this, "Not authorized to open port for sms messages");
                stopEventSource();
            }catch(InterruptedIOException iio){
                /* do nothing, this may happen if the thread is stoped while blocking connection */
            }catch(IOException ioe){
            	LogService.error(this, "Could not receive SMS from client", ioe);
            }catch(IllegalArgumentException iae){
            	LogService.error(this, "Invalid port format", iae);
            }
        }
    }
    
    public void stopEventSource(String eventShortClassName) {
        consumedEvents.removeElement(eventShortClassName);
        
        if(consumedEvents.isEmpty()){
            this.running = false;
            LogService.debug(this, "Stoped EventSource : "+getName());
            
            if(clientConnection != null){
                try{
                    /* close connection */
                    clientConnection.close();
                }catch(IOException ioe){
                	LogService.error(this, "Could not close SMS connection", ioe);
                }
            }
        }
    }

    public void stopEventSource() {
        consumedEvents.removeAllElements();
        
        this.running = false;
        LogService.debug(this, "Stoped EventSource : "+getName());
        
        if(clientConnection != null){
            try{
                /* close connection */
                clientConnection.close();
            }catch(IOException ioe){
            	LogService.error(this, "Could not close SMS connection", ioe);
            }
        }
    }
    
}
